<?php

/*
+---------------------------------------------------------------------------+
| Max Media Manager v0.3                                                    |
| =================                                                         |
|                                                                           |
| Copyright (c) 2003-2005 m3 Media Services Limited                         |
| For contact details, see: http://www.m3.net/                              |
|                                                                           |
| This program is free software; you can redistribute it and/or modify      |
| it under the terms of the GNU General Public License as published by      |
| the Free Software Foundation; either version 2 of the License, or         |
| (at your option) any later version.                                       |
|                                                                           |
| This program is distributed in the hope that it will be useful,           |
| but WITHOUT ANY WARRANTY; without even the implied warranty of            |
| MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             |
| GNU General Public License for more details.                              |
|                                                                           |
| You should have received a copy of the GNU General Public License         |
| along with this program; if not, write to the Free Software               |
| Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA |
+---------------------------------------------------------------------------+
$Id: Statistics.php 3428 2005-07-04 15:58:31Z andrew $
*/

require_once MAX_PATH . '/lib/Max.php';
 
/**
 * A factory class for creating {@link Maintenance_Statistics_Common} subclass objects.
 *
 * @package    MaxMaintenance
 * @subpackage Statistics
 * @author     Andrew Hill <andrew@m3.net>
 *
 * @TODO This class needs to be changed into a wrapper class that provides a method
 *       to kick of and run the MPS process, rather than being a factory class. This
 *       should be done when maintenance statistics is changed from its current format
 *       into a task-based format (just like the MPE is).
 */
class Maintenance_Statistics
{
    /**
     * The factory method used to instantiate a class that implements the
     * Maintenance_Statistics_Common interface.
     *
     * @param string $moduleName The name of the module class to instantiate.
     * @return Maintenance_Statistics_Common The class object that created.
     */
    function instance($moduleName)
    {
        // Instantiate the class
        $classname = $this->_deriveClassName($moduleName);
        return new $classname();
    }
    
    /**
     * A private method to derive the class name to instantiate.
     *
     * @access private
     * @param string $moduleName The name of the module class to instantiate.
     * @return string The name of the class object to create.
     */
    function _deriveClassName($moduleName)
    {
        $conf = $GLOBALS['_MAX']['CONF'];
        $filename  = $moduleName;
        $classname = 'Maintenance_Statistics_' . $moduleName;
        $includeFile = MAX_PATH . "/lib/max/Maintenance/Statistics/$filename.php";
        @include_once $includeFile;
        if (!class_exists($classname)) {
            // Unable to include the specified class file - raise error and halt
            MAX::debug('Unable to find the "' . $classname . '" class in the "' . $includeFile .
                       '" file.', PEAR_LOG_ERR);
            MAX::debug('Aborting script execution', PEAR_LOG_ERR);
            exit();
        }
        return $classname;
    }
    
}

?>
